==> Botan-1.1.10.tgz: Botan-1.1.10/src/prf_tls.cpp <==
/*************************************************
* TLS-PRF Source File                            *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/kdf.h>
#include <botan/lookup.h>
#include <botan/hmac.h>

namespace Botan {

/*************************************************
* TLS PRF                                        *
*************************************************/
OctetString TLS_PRF::derive(u32bit key_len,
                            const byte secret[], u32bit secret_len,
                            const byte seed[], u32bit seed_len) const
   {
   u32bit S1_len = (secret_len + 1) / 2,
          S2_len = (secret_len + 1) / 2;
   const byte* S1 = secret;
   const byte* S2 = secret + (secret_len - S2_len);

   OctetString key;
   key  = P_hash("MD5",   key_len, S1, S1_len, seed, seed_len);
   key ^= P_hash("SHA-1", key_len, S2, S2_len, seed, seed_len);

   return key;
   }

/*************************************************
* TLS PRF P_hash function                        *
*************************************************/
OctetString TLS_PRF::P_hash(const std::string& hash, u32bit len,
                            const byte secret[], u32bit secret_len,
                            const byte seed[], u32bit seed_len) const
   {
   SecureVector<byte> out;

   HMAC hmac(hash);
   hmac.set_key(secret, secret_len);

   SecureVector<byte> A(seed, seed_len);
   while(len)
      {
      const u32bit this_block_len = std::min(hmac.OUTPUT_LENGTH, len);

      A = hmac.process(A, A.size());

      hmac.update(A, A.size());
      hmac.update(seed, seed_len);
      SecureVector<byte> block = hmac.final();

      out.append(block, this_block_len);
      len -= this_block_len;
      }
   return out;
   }

}

==> Botan-1.3.5.tgz: Botan-1.3.5/src/prf_tls.cpp <==
/*************************************************
* TLS PRF Source File                            *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/kdf.h>
#include <botan/lookup.h>
#include <botan/hmac.h>

namespace Botan {

/*************************************************
* TLS PRF                                        *
*************************************************/
OctetString TLS_PRF::derive(u32bit key_len,
                            const byte secret[], u32bit secret_len,
                            const byte seed[], u32bit seed_len) const
   {
   u32bit S1_len = (secret_len + 1) / 2,
          S2_len = (secret_len + 1) / 2;
   const byte* S1 = secret;
   const byte* S2 = secret + (secret_len - S2_len);

   OctetString key;
   key  = P_hash("MD5",   key_len, S1, S1_len, seed, seed_len);
   key ^= P_hash("SHA-1", key_len, S2, S2_len, seed, seed_len);

   return key;
   }

/*************************************************
* TLS PRF P_hash function                        *
*************************************************/
OctetString TLS_PRF::P_hash(const std::string& hash, u32bit len,
                            const byte secret[], u32bit secret_len,
                            const byte seed[], u32bit seed_len) const
   {
   SecureVector<byte> out;

   HMAC hmac(hash);
   hmac.set_key(secret, secret_len);

   SecureVector<byte> A(seed, seed_len);
   while(len)
      {
      const u32bit this_block_len = std::min(hmac.OUTPUT_LENGTH, len);

      A = hmac.process(A);

      hmac.update(A);
      hmac.update(seed, seed_len);
      SecureVector<byte> block = hmac.final();

      out.append(block, this_block_len);
      len -= this_block_len;
      }
   return out;
   }

}
